@@ -3,12 +3,13 @@
 .CacheFiles <- NULL
 .alreadyDefinedPkgs <- NULL
 .tobeDefinedPkgs <- NULL
+.numberofRequires <- NULL
 
 .SweaveListingOptions <- list(
 Rset = list("fancyvrb" = "true", "escapechar" = "`",
         "language" = "R", "basicstyle" = "{\\color{Rcolor}\\small}",
         "keywordstyle" = "{\\bf\\color{Rcolor}}",
-        "commentstyle" = "{\\color{Rcomment}\\ttfamily\\itshape}",
+        "commentstyle" = "{\\color{Rcommentcolor}\\ttfamily\\itshape}",
         "literate" = paste("{<-}{{$\\leftarrow$}}2",
                            "{<<-}{{$\\twoheadleftarrow$}}2",#"%\n",
                            "{~}{{$\\sim$}}1", "{<=}{{$\\leq$}}2",#"%\n",
@@ -23,10 +24,16 @@ Rdset = list("fancyvrb" = "true", "language" = "Rd",
              "basicstyle" = "{\\color{black}\\footnotesize}",
                "commentstyle" = "{\\ttfamily\\itshape}",
                "alsolanguage" = "R"),
+Rin = list("style" = "Rstyle", "fancyvrb" = "true",
+           "basicstyle" = "\\color{Rcolor}\\small"),
+Rout = list("fancyvrb" = "false", "basicstyle" = "\\color{Routcolor}\\small"),
+Rcode = list("style" = "Rstyle", "fancyvrb" = "true",
+             "fontshape"= "sl", "basicstyle" = "\\color{Rcolor}"),
 Rcolor  = c(0,0.5,0.5),
-Rbcolor  = c(0,0.6,0.6),
-Rout     = c(0.461,0.039,0.102),
-Rcomment = c(0.101,0.043,0.432),
+RRecomdcolor  = c(0,0.6,0.4),
+Rbcolor       = c(0,0.6,0.6),
+Routcolor     = c(0.461,0.039,0.102),
+Rcommentcolor = c(0.101,0.043,0.432),
 pkv = "2.0.2",
 pkg = "distr",
 Keywordstyle = "{\\bf}",
@@ -36,7 +43,12 @@ intermediate = TRUE,
 inSweave = FALSE,
 fromRForge = TRUE,
 base.url = paste("http://r-forge.r-project.org/plugins/scmsvn/viewcvs.php/",
-                            "*checkout*/pkg/", sep = "")
+                            "*checkout*/pkg/", sep = ""),
+addRset = TRUE,
+addRdset = TRUE,
+addRinset = TRUE,
+addRoutset = TRUE,
+addRcodeset = TRUE
 )
 
 SweaveListingOptions <- function(...) {
